package POWER::Utils::HexDump;

use version; $VERSION = qv('0.0.3');

use warnings;
use strict;
use Carp;

use List::Util qw( min max );

use Perl6::Export::Attrs;


sub HexDump2Bin :Export(:DEFAULT) {
    local $_ = join "", @_;
    # убрать разделитель внутри hex в mc
    s/\x83\x20//g;                        
    # слить вместе столбцы hex
    s/(?<=[0-9a-fA-F]) (?=[0-9a-fA-F])//g;
    if (/^\s*\S+\s+(?:[0-9a-fA-F]{2})+(?:\s|$)/) {
        # если второе поле состоит из hex, то берем его ...
        s/^\s*\S+\s+(\S+).*\n?/$1/mg;
    } elsif (/^\s*(?:[0-9a-fA-F]{2})+(?:\s|$)/) {
        # ... иначе если первое поле состоит из hex, то берем его ...
        s/^\s*(\S+).*\n?/$1/mg;
    } else {
        # ... иначе все плохо
        croak "Unknown packet format:\n---cut---\n$_\n---cut---\n";
    }
    # контроль формата
    croak "Internal error: bad format $_\n" unless /^(?:[0-9a-fA-F]{2})+$/;
    return pack "H*", $_;
}

sub Bin2HexDump :Export(:DEFAULT) {
    (local $_, my %o) = (shift, offs=>1,ascii=>1,group=>4,uc=>1,groups=>8,@_);
    # разбить на строчки и на группы внутри строчек
    my $groups = 0;
    $_ = join "", map {($o{uc} ? uc($_) : lc($_)) . ((($groups+=1)%=$o{groups})
        ? " " : "\n")} unpack "(H$o{group})*", $_||"";
    # дополнить последнюю строку пробелами если она не полная
    s/^(.+)\Z/$1 . " " x ($o{groups}-1 + $o{group}*$o{groups} - length($1))/em;
    # дописать ascii и offs
    my ($offs, $count) = (0, $o{group}*$o{groups}/2);
    join "", 
        map {$o{offs} ? sprintf("0x%04X  %s",($offs+=$count)-$count, $_) : $_}
        map {s/[^\x20-\x7E\xC0-\xFE]/./g;"$_\n"}
        map {$o{ascii} ? $_."   ".(pack"H*",join"",split) : $_} split /\n/;
}

sub Bin2HexDiff :Export(:DEFAULT) {
    my ($bin1, $bin2, %o) = @_;
    # enable ansi colors only for terminals by default
    $o{ansi} = -t select() unless defined $o{ansi};
    my $diff = "";
    # разбить дампы по строчкам
    my @txt1 = split /\n/, Bin2HexDump($bin1, %o);
    my @txt2 = split /\n/, Bin2HexDump($bin2, %o);
    # обработать общие строчки
    for (0 .. min($#txt1,$#txt2)) {
        # не дублировать строчку, если она не отличается
        $diff .= "  ".$txt1[$_]."\n", next if $txt1[$_] eq $txt2[$_];
        # раскрасить отличия, если нужно
        if ($o{ansi}) {
            # посимвольная обработка
            my @l1 = split //, $txt1[$_];
            my @l2 = split //, $txt2[$_];
            # сравнить только общие символы
            for (grep {$l1[$_] ne $l2[$_]} 0 .. min($#l1,$#l2)) {
                $l1[$_] = "\e[31m".$l1[$_]."\e[37m";
                $l2[$_] = "\e[32m".$l2[$_]."\e[37m";
            }
            $txt1[$_] = join "", @l1;
            $txt2[$_] = join "", @l2;
        }
        # вывести отличающиеся строки
        $diff .= "- ".$txt1[$_]."\n";
        $diff .= "+ ".$txt2[$_]."\n";
    }
    # дописать не общие строчки (остаток, существующий только в txt1 или txt2)
    $diff .= ($txt1[$_] ? "- " : "+ ").$txt1[$_].$txt2[$_]."\n"
        for min($#txt1,$#txt2)+1 .. max($#txt1,$#txt2);
    return $diff;
}


1; # Magic true value required at end of module
__END__

=encoding utf-8

=head1 NAME

POWER::Utils::HexDump - work with hex dumps


=head1 VERSION

This document describes POWER::Utils::HexDump version 0.0.3


=head1 SYNOPSIS

 use POWER::Utils::HexDump;

 read SOCK, $buf1, 1024;
 print Bin2HexDump($buf1);
 print SOCK HexDump2Bin(`cat tcpdump.log`);
 read SOCK, $buf2, 1024;
 print Bin2HexDiff($buf1, $buf2);


=head1 DESCRIPTION

Helpers for working with hex dumps: convert between hex dump and byte
streams, diff hex dumps. Understand a number of hex dump formats.


=head1 EXPORTS

=over

=item :DEFAULT

Bin2HexDump() HexDump2Bin() Bin2HexDiff()


=back

=head1 INTERFACE

=over

=item B<HexDump2Bin>( $hex_dump )

Convert hex dump (like Midnight Commander's hex viewer output OR
tcpdump output, etc.) to binary string (automatically detect input format and
ignore offset at start and ascii text at end). Example of input formats:

 10D80  01 00 00 08 | 04 00 AF 05 | 04 00 21 F9 | 04 20 03 00  ....╞
 10D90  00 00 2C 00 | 00 20 00 01 | 00 01 00 00 | 08 04 00 AF  .., .

 0x0000   c0a8 0201 0001 009b 831d 0000 dc02 e63e   ...............>
 0x0010   4500 0054 3ee1 4000 8001 3674 c0a8 0202   E..T>.@...6t....

=item B<Bin2HexDump>( $binary [, %options] )

Convert binary string to hex dump. Format of hex dump can be configured using
%options. Default value for %options is (offs=>1, ascii=>1, group=>4,
uc=>1, groups=>8):

 offs   : should be true to include offset at start of each string
 ascii  : should be true to include ascii dump at end of each string
 group  : how many hex digits should be in one group (usually 2 or 4)
 groups : how many such groups should be in one string
 uc     : true to uc() hex digits, false to lc() hex digits

=item B<Bin2HexDiff>( $binary1, $binary2 [, %options] )

Convert two binary strings using given %options to two hex dumps, compare
these hex dumps and return diff. Bin2HexDiff() use additional option:

 ansi   : true to return ANSI-colored diff; if not defined then
          standart output will be checked - if it is on TTY then use
	  ANSI-colored diff


=back

=head1 DIAGNOSTICS

=over

=item C<< Unknown packet format:
---cut---
...
---cut---
>>

HexDump2Bin() unable to understand format of your hex dump.


=item C<< Internal error: bad format ... >>

If you see this error - you find a bug in this module.


=back

=head1 CONFIGURATION AND ENVIRONMENT

POWER::Utils::HexDump requires no configuration files or environment variables.


=head1 DEPENDENCIES

=over

=item version

=item Perl6::Export::Attrs


=back

=head1 INCOMPATIBILITIES

None reported.


=head1 BUGS AND LIMITATIONS

No bugs have been reported.

Please report any bugs or feature requests to
C<bug-power-utils-hexdump@rt.cpan.org>, or through the web interface at
L<http://rt.cpan.org>.


=head1 AUTHOR

Alex Efros  <powerman@asdfgroup.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) 2005, Alex Efros <powerman@asdfgroup.com>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
